// appmanager-sample.js
// 
// In this sample application information will be listed; The 
// document and application will be lanched; Also, async operation
// will be canceled.
//

//SAPI Error Codes
// 0    - Success
// 1000 - Incorrect argument
// 1001 - UnknownArgumentName
// 1002 - BadArgumentType
// 1003 - MissingArgument
// 1004 - ServiceNotSupported
// 1005 - ServiceInUse
// 1006 - ServiceNotReady
// 1007 - NoMemory
// 1008 - HardwareNotAvailable
// 1009 - ServerBusy
// 1010 - EntryExists
// 1011 - AccessDenied
// 1012 - NotFound
// 1013 - UnknownFormat
// 1014 - GeneralError
// 1015 - CancelSuccess
// 1016 - ServiceTimedOut
// 1017 - File already exist

// Declare the service object 
var so;

// imgid for callback1 function
var imgid_callback1;

// imgid for callback2 function
var imgid_callback2;

// id of the div used to display information
const DIV_ID = 'appman';

// Called from onload()
function setup()
{
  try {
    so = device.getServiceObject("Service.AppManager", "IAppManager");
    console.info("setup: so: %s", so);
  }
  catch (e) {   
    alert('<setup> ' +e);
  }
}

// Get application list without filter
function getList(imgId)
{
  // 'UserInstalledPackage'
  // 'Application'
  var msg = "Please enter the list type ['UserInstalledPackage' or 'Application']";
  var listType = prompt(msg, "Application");
  
  if(listType == "" || listType == null)
  return;
  
  var criteria = new Object();
  criteria.Type = listType;  
  
  var result;
  try {
    result = so.IAppManager.GetList(criteria);
    if(!checkError("IAppManager::GetList",result,DIV_ID,imgId)) {        
      document.getElementById(DIV_ID).innerHTML = showIterableList(result.ReturnValue);
    }    
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("getList: " + e);     
  }
  
  if( result != null && result != undefined)
    result.close();
}

// Get application list with filter
function getFilteredList(imgId)
{
  var mimeType = prompt("Please enter the mimetype","image/jpeg");
  if(mimeType == "" || mimeType == null)
    return;
  
  var filter = new Object();
  filter.MimeType = mimeType;
  
  var criteria = new Object();
  criteria.Type = "Application";
  criteria.Filter = filter;
  
  try {
    var result = so.IAppManager.GetList(criteria);
    if(!checkError("IAppManager::GetFilteredList",result,DIV_ID,imgId)) {
      document.getElementById(DIV_ID).innerHTML = showIterableList(result.ReturnValue);
    }
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("getFilteredList: " + e);
  }
  
  if( result != null && result != undefined)
    result.close();
}

// Launch document
function launchDoc(imgId)
{
  var docPath = prompt("Please enter the path", "c:\\data\\image.jpg");
  
  if(docPath == "" || docPath == null)
    return;
  
  var mimeType = prompt("Please enter the mimetype", "image/jpeg");
  
  if(mimeType == null)
    return;

  var docs = new Object();
  docs.DocumentPath = docPath;
  
  var criteria = new Object();
  criteria.Document = docs;
  criteria.MimeType = mimeType;

  try {
    var result = so.IAppManager.LaunchDoc(criteria);
    checkError("IAppManager::LaunchDoc",result,DIV_ID,imgId);
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("launchDoc: " + e);
  } 
}

// Launch document (Async operation)
function launchDocAsync(imgId)
{
  var docPath = prompt("Please enter the path", "c:\\data\\image.jpg");
  
  if(docPath == "" || docPath == null)
    return;
  
  var mimeType = prompt("Please enter the mimetype", "image/jpeg");
  
  if(mimeType == null)
    return;
  
  var docs = new Object();
  docs.DocumentPath = docPath;
  
  var criteria = new Object();
  criteria.Document = docs;
  criteria.MimeType = mimeType;
  
  try {
    imgid_callback1 = imgId;
    var result = so.IAppManager.LaunchDoc(criteria,callback1);
    if(!checkError("IAppManager::launchDocAsync",result,DIV_ID,imgId))
      showIMG(imgId,"");
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("launchDocAsync: " + e);
  } 
}

// Launch application
function launchApp(imgId)
{
  // get the appid by calling the getList()
  var appid = prompt("Please enter the AppId", "s60uid://0x101f4cce");
  
  if(appid == "" || appid == null)
    return;
  
  var opt = new Object();
  opt.Mode = "Standalone";
  
  var criteria = new Object();
  criteria.ApplicationID = appid;
  criteria.Option = opt;  
  
  try {
    var result = so.IAppManager.LaunchApp(criteria);
    checkError("IAppManager::launchApp",result,DIV_ID,imgId);
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("launchApp: " + e);
  } 
}

// Launch application (Async operation)
function launchAppAsync(imgId)
{
  // get the appid by calling the getList()
  var appid = prompt("Please enter the AppId", "s60uid://0x101f4cce");
  
  if(appid == "" || appid == null)
    return;
  
  var opt = new Object();
  opt.Mode = "Standalone";
  
  var criteria = new Object();
  criteria.ApplicationID = appid;
  criteria.CmdLine = "";
  criteria.Option = opt;  
  
  try {
    imgid_callback2 = imgId;
    var result = so.IAppManager.LaunchApp(criteria,callback2);
    if(!checkError("IAppManager::launchAppAsync",result,DIV_ID,imgId))
    showIMG(imgId, "");
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("launchApp: " + e);
  }
}

// Cancle Launch document 
function cancelLaunchDocAsync(imgId)
{
  var docPath = prompt("Please enter the path", "c:\\data\\image.jpg");
  
  if(docPath == "" || docPath == null)
    return;
  
  var mimeType = prompt("Please enter the mimetype", "image/jpeg");
  
  if(mimeType == null)
    return;
  
  var docs = new Object();
  docs.DocumentPath = docPath;
  
  var criteria = new Object();
  criteria.Document = docs;
  criteria.MimeType = mimeType;
  
  try {
    imgid_callback1 = imgId;
    var result = so.IAppManager.LaunchDoc(criteria,callback1);
    if(!checkError("IAppManager::cancelLaunchDocAsync",result,DIV_ID,imgId)) {
      showIMG(imgId, "");
      var criteria2 = new Object();
      criteria2.TransactionID = result.TransactionID;
      var cresult = so.IAppManager.Cancel(criteria2);
      checkError("IAppManager::cancelLaunchDocAsync",cresult,DIV_ID,imgId);
    }
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("cancelLaunchDocAsync: " + e);
  } 
}

// Cancel Launch application 
function cancelLaunchAppAsync(imgId)
{
  // get the appid by calling the getList()
  var appid = prompt("Please enter the AppId", "s60uid://0x101f4cce");
  
  if(appid == "" || appid == null)
    return;
  
  var opt = new Object();
  opt.Mode = "Standalone";
  
  var criteria = new Object();
  criteria.ApplicationID = appid;
  criteria.Option = opt;  
  
  try {
    imgid_callback2 = imgId;
    result = so.IAppManager.LaunchApp(criteria,callback2);
    if(!checkError("IAppManager::cancellaunchAppAsync",result,DIV_ID,imgId)) {
      showIMG(imgId, "");
      var criteria2 = new Object();
      criteria2.TransactionID = result.TransactionID;
      var cresult = so.IAppManager.Cancel(criteria2);
      checkError("IAppManager::cancelLaunchAppAsync",cresult,DIV_ID,imgId);
    }
  }
  catch (e) {
    showIMG(imgId,"no");
    alert ("cancelLaunchAppAsync: " + e);
  }
}

// This is the asynchronous callback handler 
function callback1(transId, eventCode, result)
{
  console.info("callback1: transId: %d  eventCode: %d result.ErrorCode: %d", transId, eventCode, result.ErrorCode);
  if(!checkError("IAppManager::launchAppAsync",result,DIV_ID,imgid_callback1)) {
    document.getElementById(DIV_ID).innerHTML = "The lanched document died."
  }
}

// This is the asynchronous callback handler 
function callback2(transId, eventCode, result)
{
  console.info("callback2: transId: %d  eventCode: %d result.ErrorCode: %d", transId, eventCode, result.ErrorCode);
  if(!checkError("IAppManager::launchAppAsync",result,DIV_ID,imgid_callback2)) {
    document.getElementById(DIV_ID).innerHTML = "The lanched application died."
  }
}